/*
 Magnet Push-Pin with Custom Graphic
 Copyright 2025 Tyler Tork
 
 This model is a pushpin-style magnet you can customize with a top graphic (SVG file) or shape (STL file). The parameters let you specify the sizes of the magnet you are using and the dimensions you want to custom artwork to appear, among other things.
 
 The default parameter values are intended to work with a stack of two 5mm magnets you can get from Amazon.com: https://amzn.to/3PvZZDm (affiliate link). But you can use any cylindrical magnet.
 
  A complete guide for configuring this model can be found at [URL here]

 Licensed under Creative Commons - Attribution - Non-Commercial - Share Alike
    https://creativecommons.org/licenses/by-nc-sa/4.0/
 */
/* [Import settings] */
filename = "default.svg";
top_width = 12; // 0.1
top_offset = [0,0,0]; // 0.1

/* [if 2D import] */
pattern_height = .3; // 0.01
platform_height = 1; // 0.01
outline_style = "hull"; // [hull,fill,disk]
outline_enlarge = 0; // 0.1
// change threshold to adjust conversion of PNG file to outline.
trace_threshold = 50; // [1:99]
// reverse dark and light in PNG file trace
trace_invert = false;
/* [if 3D import] */
rotation = 0;
z_scale = 1; // [.05:.01:1]

/* [Post dimensions] */
// Add magnets during printing and completely enclose them behind this thick a base?
magnet_embed = 0; // 0.01
magnet_diameter = 5.6;
//default=stack of 2 little round magnets
magnet_height = 3.67;
post_height = 10;
post_diameter = 6;
wall_thickness = 1.2;
// if bottom is open, whether to give magnet hole a pointy ceiling
peak_in_magnet_hole = false;

/* [Advanced] */
section = false;

/* [Hidden] */

debug = false;
// the following parameters belong in the "if 3D import" section but were moved to here because they don't work as well as I'd hoped in the general case.
support_cone_diameter = 0; // 0.1
support_cone_z_adjust = 0; // 0.1

include <BOSL2/std.scad>
EPS = 0.01;
$fa=2; $fs=.4;
main_color = [1,1,.6];
line_color = [.2,.2,1];

rMagnet = magnet_diameter/2;
htPoint = max(0,rMagnet-1);
rPost = post_diameter/2;
rBase = max(rPost, rMagnet + wall_thickness);
filetype = fileExt(filename);
isBitmap = (filetype == "png");
is2D = (isBitmap || filetype == "svg" || filetype == "dxf") && filename != "default.svg";
is3D = filetype == "stl" || filetype == "obj" || filetype == "off" || filetype == "3mf";

function fileExt(name) =
	let(parts = str_split(name,"."))
	downcase(parts[len(parts)-1]);
	
module post(extraheight=EPS) {
	htMag = magnet_height+max(0,magnet_embed);
	ptsPost = [
		[0,0],
		[rBase, 0],
		[rBase, htMag+1],
		[rPost, htMag+1+abs(rBase-rPost)],
		[rPost, post_height+extraheight],
		[0,post_height+extraheight]
	];
	echo(ptsPost);
	color(main_color)
	rotate_extrude()
	difference() {
		polygon(ptsPost);
		
		translate([-.5, post_height+extraheight])
		square([max(rPost,rBase)+1, 100]);
	}
}

module postHole() {
	color("gold")
	if (magnet_embed <= 0) {
		down(EPS)
		cylinder(magnet_height+EPS, d=magnet_diameter);
		if (peak_in_magnet_hole) {
			up(magnet_height-EPS)
			cylinder(htPoint+EPS, r1=htPoint+EPS, r2=0);
		}
	} else {
		up(magnet_embed)
		cylinder(magnet_height, d=magnet_diameter);
	}
}

module top() {
	difference() {
		up(post_height)
		up(top_offset.z)
		button()
		translate([top_offset.x, top_offset.y, 0])
		resize([top_width, 0, 0], auto=true) {
			if (isBitmap) {
				projection(true)
				down((trace_invert?1:-1)*trace_threshold)
				resize([0,0,100], true)
				surface(file=filename, invert=!trace_invert, center=true);
			} else if (is2D) {
				import(filename);
			} else if (is3D) {
				xrot(rotation) import(filename);
			} else {
				r = top_width/2;
				pts = [
					[0,-r], [r,0], [r,3], [0,3]
				];
				color(main_color)
				rotate_extrude() polygon(pts);
			}
		}
		// remove anything below the XY plane.
		down(top_width*2) cube(top_width*4, center=true);
	}
}

module fill2d() {
	if (outline_style=="hull") {
		offset(r=outline_enlarge) hull() children();
	} else if (outline_style=="fill") {
		fill() offset(r=outline_enlarge) children();
	} else {
		union() {
			circle(d=top_width+2*outline_enlarge);
			hull() children();
		}
	}
}

// given a child which is the import file, centered based on the offset, make it into the 3D object that's the top of the pin.
module button() {
	if (is2D) {
		color(line_color)
		up(platform_height)
		linear_extrude(height=pattern_height)
		children();
		
		color(main_color) {
		linear_extrude(height=platform_height)
		fill2d()
		children();
		
		scale([1,1,-1])
		linear_extrude(height=top_width/2, scale=0)
		fill2d()
		children();
		}
	} else if (is3D) {
		scale([1,1,z_scale]) children();
		diamSC = support_cone_diameter;
		if (diamSC > 0) {
			up(support_cone_z_adjust)
			scale([1,1,-1])
			intersection() {
				cylinder(h=diamSC/2, d1=diamSC, d2=.1);
				down(EPS) linear_extrude(height=diamSC/2+EPS*2, scale=.5) projection() children();
			}
		}
	} else {
		children();
	}
}

if (debug) {
up(magnet_embed+.08) cylinder(h=magnet_height-.16, d=magnet_diameter-.16);
color("pink", .3) post();
} else {
	render()
	difference() {
		union() { post(); top(); }
		postHole();
		if (section && $preview) {
			color("gold") down(EPS) cube(max(100,post_height*5));
		}
	}
	if (section && $preview) {
		%up(magnet_embed+.08) cylinder(h=magnet_height-.16, d=magnet_diam-.16);
	}
}